@@ -17,11 +17,8 @@
               "Water table depth" = "WTD")
   }
   TYPES = c(TYPES, 
-            "Plant extraction from soil"= "PlantExtraction")
-  if(transpirationMode=="Sperry") {
-    TYPES = c(TYPES, 
-              "Hydraulic redistribution" = "HydraulicRedistribution")
-  }
+            "Plant extraction from soil"= "PlantExtraction",
+            "Hydraulic redistribution" = "HydraulicRedistribution")
   return(TYPES)
 }
 .getStandPlotTypes<-function(model = "pwb") {
@@ -29,6 +26,7 @@
             "Ground-level irradiance" = "GroundIrradiance")
   if(model %in% c("growth", "fordyn")) {
     TYPES = c(TYPES, 
+              "Carbon balance" = "CarbonBalance",
               "Biomass balance" = "BiomassBalance")
   }
   return(TYPES)
@@ -159,11 +157,6 @@
            "Sapwood starch concentration" = "StarchSapwood",
            "Sugar transport" = "SugarTransport",
            "Root exudation" = "RootExudation")
-  if(transpirationMode=="Sperry") {
-    TYPES = c(TYPES,
-              "Leaf osmotic potential at full turgor" = "LeafPI0",
-              "Stem osmotic potential at full turgor" = "StemPI0")
-  }
   return(TYPES)
 }
 
@@ -187,7 +180,8 @@
 }
 
 .getUniqueDailyGROWTHPlotTypes<-function(transpirationMode = "Granier"){
-  TYPES = c("Biomass balance" = "BiomassBalance",
+  TYPES = c("Carbon balance" = "CarbonBalance",
+            "Biomass balance" = "BiomassBalance",
             .getLabileGROWTHPlotTypes(transpirationMode),
             .getCohortBiomassBalanceGROWTHPlotTypes(transpirationMode),
             .getStructuralGROWTHPlotTypes(transpirationMode),
@@ -800,6 +794,19 @@
   if(!is.null(summary.freq)) df = .temporalSummary(df, summary.freq, mean, na.rm=TRUE)
   return(.multiple_dynamics(as.matrix(df), ylab = ylab, ylim = ylim))
 }
+.plot_carbon<-function(CarbonBalance, type,  
+                        dates = NULL, 
+                        xlim = NULL, ylim=NULL, xlab=NULL, ylab=NULL, 
+                        summary.freq = NULL, ...) {
+  df<-as.data.frame(CarbonBalance)
+  df[,2] = - df[,2]
+  df[,3] = - df[,3]
+  names(df)<-c("Gross primary production", "Maintenance respiration","Synthesis respiration", "Net primary production")
+  if(is.null(ylab))  ylab = expression(gC%.%m^{-2})    
+  if(!is.null(dates)) df = df[row.names(df) %in% as.character(dates),,drop = FALSE]
+  if(!is.null(summary.freq)) df = .temporalSummary(df, summary.freq, mean, na.rm=TRUE)
+  return(.multiple_dynamics(as.matrix(df), ylab = ylab, ylim = ylim))
+}
 .sumSubdailyBySpecies<-function(OM, spnames) {
   if(ncol(OM)>2) OM = cbind(OM[,1],t(apply(OM[,-1],1, tapply, spnames, sum, na.rm=T)))
   else colnames(OM)[2] = spnames[1]
