@@ -336,7 +336,8 @@ double coarseRootSoilVolumeFromConductance(double Kmax_rootxylem, double VCroot_
   double vol = 0.0;
   double f = frv(vol, B, v, ax, ra);
   while(std::abs(f)>fTol) {
-    if(f>0.0) {
+    // Rcout<<vol<<"\n";
+    if((f > 0.0)) {
       vol += step; 
     } else {
       vol -= step;
@@ -347,7 +348,7 @@ double coarseRootSoilVolumeFromConductance(double Kmax_rootxylem, double VCroot_
   // for(int j=0;j<numLayers;j++) {
     // Rcout<<j<<" "<<ax[j]<<" "<<sqrt(vol)*ra[j]<<" "<<((d[j]/1000.0)*M_PI*pow(sqrt(vol)*ra[j],2.0))<<"\n";
   // }
-  return(vol);
+  return(std::max(0.25,vol));
 }
 
 /**
